@@ -17,6 +17,9 @@
  */
 
 /* M_PI is defined in math.h in the case of Microsoft Visual C++ */
+#if defined(_WIN32)
+#define _USE_MATH_DEFINES
+#endif
 
 #include <cairommconfig.h>
 #include <cairomm/context.h>
@@ -43,7 +46,7 @@ Context::Context(const RefPtr<Surface>& target)
 
 RefPtr<Context> Context::create(const RefPtr<Surface>& target)
 {
-  return RefPtr<Context>(new Context(target));
+  return make_refptr_for_instance<Context>(new Context(target));
 }
 
 Context::Context(cairo_t* cobject, bool has_reference)
@@ -151,25 +154,6 @@ void Context::set_line_join(LineJoin line_join)
   check_object_status_and_throw_exception(*this);
 }
 
-#ifndef CAIROMM_DISABLE_DEPRECATED
-void Context::set_dash(std::valarray<double>& dashes, double offset)
-{
-  std::vector<double> v(dashes.size());
-  for(size_t i = 0; i < dashes.size(); ++i)
-    v[i] = dashes[i];
-
-  set_dash(v, offset);
-}
-
-void Context::set_dash(std::vector<double>& dashes, double offset)
-{
-  cairo_set_dash(cobj(),
-    (dashes.empty() ? nullptr : &dashes[0]),
-    dashes.size(), offset);
-  check_object_status_and_throw_exception(*this);
-}
-#endif //CAIROMM_DISABLE_DEPRECATED
-
 void Context::set_dash(const std::valarray<double>& dashes, double offset)
 {
   std::vector<double> v(dashes.size());
@@ -182,14 +166,14 @@ void Context::set_dash(const std::valarray<double>& dashes, double offset)
 void Context::set_dash(const std::vector<double>& dashes, double offset)
 {
   cairo_set_dash(cobj(),
-    (dashes.empty() ? nullptr : &dashes[0]),
+    (dashes.empty() ? 0 : &dashes[0]),
     dashes.size(), offset);
   check_object_status_and_throw_exception(*this);
 }
 
 void Context::unset_dash()
 {
-  cairo_set_dash(cobj(), nullptr, 0, 0.0);
+  cairo_set_dash(cobj(), NULL, 0, 0.0);
   check_object_status_and_throw_exception(*this);
 }
 
@@ -229,80 +213,36 @@ void Context::transform(const Matrix& matrix)
   check_object_status_and_throw_exception(*this);
 }
 
-void Context::transform(const cairo_matrix_t& matrix)
-{
-  cairo_transform(cobj(), &matrix);
-  check_object_status_and_throw_exception(*this);
-}
-
 void Context::set_matrix(const Matrix& matrix)
 {
   cairo_set_matrix(cobj(), &matrix);
   check_object_status_and_throw_exception(*this);
 }
 
-void Context::set_matrix(const cairo_matrix_t& matrix)
-{
-  cairo_set_matrix(cobj(), &matrix);
-  check_object_status_and_throw_exception(*this);
-}
-
 void Context::set_identity_matrix()
 {
   cairo_identity_matrix(cobj());
   check_object_status_and_throw_exception(*this);
 }
 
-#ifndef CAIROMM_DISABLE_DEPRECATED
-void Context::user_to_device(double& x, double& y)
-{
-  const Context* constThis = this;
-  constThis->user_to_device(x, y);
-}
-#endif //CAIROMM_DISABLE_DEPRECATED
-
 void Context::user_to_device(double& x, double& y) const
 {
   cairo_user_to_device(const_cast<cobject*>(cobj()), &x, &y);
   check_object_status_and_throw_exception(*this);
 }
 
-#ifndef CAIROMM_DISABLE_DEPRECATED
-void Context::user_to_device_distance(double& dx, double& dy)
-{
-  const Context* constThis = this;
-  constThis->user_to_device_distance(dx, dy);
-}
-#endif //CAIROMM_DISABLE_DEPRECATED
-
 void Context::user_to_device_distance(double& dx, double& dy) const
 {
   cairo_user_to_device_distance(const_cast<cobject*>(cobj()), &dx, &dy);
   check_object_status_and_throw_exception(*this);
 }
 
-#ifndef CAIROMM_DISABLE_DEPRECATED
-void Context::device_to_user(double& x, double& y)
-{
-  const Context* constThis = this;
-  constThis->device_to_user(x, y);
-}
-#endif //CAIROMM_DISABLE_DEPRECATED
-
 void Context::device_to_user(double& x, double& y) const
 {
   cairo_device_to_user(const_cast<cobject*>(cobj()), &x, &y);
   check_object_status_and_throw_exception(*this);
 }
 
-#ifndef CAIROMM_DISABLE_DEPRECATED
-void Context::device_to_user_distance(double& dx, double& dy)
-{
-  const Context* constThis = this;
-  constThis->device_to_user_distance(dx, dy);
-}
-#endif //CAIROMM_DISABLE_DEPRECATED
-
 void Context::device_to_user_distance(double& dx, double& dy) const
 {
   cairo_device_to_user_distance(const_cast<cobject*>(cobj()), &dx, &dy);
@@ -518,7 +458,7 @@ void Context::copy_clip_rectangle_list(std::vector<Rectangle>& rectangles) const
   cairo_rectangle_list_destroy(c_list);
 }
 
-void Context::select_font_face(const std::string& family, FontSlant slant, FontWeight weight)
+void Context::select_font_face(const std::string& family, ToyFontFace::Slant slant, ToyFontFace::Weight weight)
 {
   cairo_select_font_face(cobj(), family.c_str(),
           static_cast<cairo_font_slant_t>(slant),
@@ -538,24 +478,12 @@ void Context::set_font_matrix(const Matrix& matrix)
   check_object_status_and_throw_exception(*this);
 }
 
-void Context::set_font_matrix(const cairo_matrix_t& matrix)
-{
-  cairo_set_font_matrix(cobj(), &matrix);
-  check_object_status_and_throw_exception(*this);
-}
-
 void Context::get_font_matrix(Matrix& matrix) const
 {
   cairo_get_font_matrix(const_cast<cobject*>(cobj()), &matrix);
   check_object_status_and_throw_exception(*this);
 }
 
-void Context::get_font_matrix(cairo_matrix_t& matrix) const
-{
-  cairo_get_font_matrix(const_cast<cobject*>(cobj()), &matrix);
-  check_object_status_and_throw_exception(*this);
-}
-
 void Context::set_font_options(const FontOptions& options)
 {
   cairo_set_font_options(cobj(), options.cobj());
@@ -578,7 +506,7 @@ RefPtr<ScaledFont> Context::get_scaled_font()
 {
   auto font = cairo_get_scaled_font(cobj());
   check_object_status_and_throw_exception(*this);
-  return RefPtr<ScaledFont>(new ScaledFont(font, false /* does not have reference */));
+  return make_refptr_for_instance<ScaledFont>(new ScaledFont(font, false /* does not have reference */));
 }
 
 void Context::show_text(const std::string& utf8)
@@ -593,9 +521,9 @@ void Context::show_text_glyphs(const std::string& utf8,
                                TextClusterFlags cluster_flags)
 {
   cairo_show_text_glyphs(cobj(), utf8.c_str(), utf8.size(),
-                         (glyphs.empty() ? nullptr : &glyphs[0]),
+                         (glyphs.empty() ? 0 : &glyphs[0]),
                          glyphs.size(),
-                         (clusters.empty() ? nullptr : &clusters[0]),
+                         (clusters.empty() ? 0 : &clusters[0]),
                          clusters.size(),
                          static_cast<cairo_text_cluster_flags_t>(cluster_flags));
   check_object_status_and_throw_exception(*this);
@@ -604,7 +532,7 @@ void Context::show_text_glyphs(const std::string& utf8,
 void Context::show_glyphs(const std::vector<Glyph>& glyphs)
 {
   cairo_show_glyphs(cobj(),
-    const_cast<cairo_glyph_t*>((glyphs.empty() ? nullptr : &glyphs[0])),
+    const_cast<cairo_glyph_t*>((glyphs.empty() ? 0 : &glyphs[0])),
     glyphs.size());
   check_object_status_and_throw_exception(*this);
 }
@@ -613,14 +541,14 @@ RefPtr<FontFace> Context::get_font_face()
 {
   auto cfontface = cairo_get_font_face(cobj());
   check_object_status_and_throw_exception(*this);
-  return RefPtr<FontFace>(new FontFace(cfontface, false /* does not have reference */));
+  return make_refptr_for_instance<FontFace>(new FontFace(cfontface, false /* does not have reference */));
 }
 
 RefPtr<const FontFace> Context::get_font_face() const
 {
   auto cfontface = cairo_get_font_face(const_cast<cobject*>(cobj()));
   check_object_status_and_throw_exception(*this);
-  return RefPtr<const FontFace>(new FontFace(cfontface, false /* does not have reference */));
+  return make_refptr_for_instance<const FontFace>(new FontFace(cfontface, false /* does not have reference */));
 }
 
 void Context::get_font_extents(FontExtents& extents) const
@@ -644,7 +572,7 @@ void Context::get_text_extents(const std::string& utf8, TextExtents& extents) co
 void Context::get_glyph_extents(const std::vector<Glyph>& glyphs, TextExtents& extents) const
 {
   cairo_glyph_extents(const_cast<cobject*>(cobj()),
-                      const_cast<cairo_glyph_t*>(glyphs.empty() ? nullptr : &glyphs[0]),
+                      const_cast<cairo_glyph_t*>(glyphs.empty() ? 0 : &glyphs[0]),
                       glyphs.size(), &extents);
   check_object_status_and_throw_exception(*this);
 }
@@ -658,12 +586,12 @@ void Context::text_path(const std::string& utf8)
 void Context::glyph_path(const std::vector<Glyph>& glyphs)
 {
   cairo_glyph_path(cobj(),
-    const_cast<cairo_glyph_t*>(glyphs.empty() ? nullptr : &glyphs[0]),
+    const_cast<cairo_glyph_t*>(glyphs.empty() ? 0 : &glyphs[0]),
     glyphs.size());
   check_object_status_and_throw_exception(*this);
 }
 
-Operator Context::get_operator() const
+Context::Operator Context::get_operator() const
 {
   const auto result =
     static_cast<Operator>(cairo_get_operator(const_cast<cobject*>(cobj())));
@@ -677,19 +605,19 @@ static RefPtr<Pattern> get_pattern_wrapper (cairo_pattern_t* pattern)
   switch (pattern_type)
   {
     case CAIRO_PATTERN_TYPE_SOLID:
-      return RefPtr<SolidPattern>(new SolidPattern(pattern, false /* does not have reference */));
+      return make_refptr_for_instance<SolidPattern>(new SolidPattern(pattern, false /* does not have reference */));
       break;
     case CAIRO_PATTERN_TYPE_SURFACE:
-      return RefPtr<SurfacePattern>(new SurfacePattern(pattern, false /* does not have reference */));
+      return make_refptr_for_instance<SurfacePattern>(new SurfacePattern(pattern, false /* does not have reference */));
       break;
     case CAIRO_PATTERN_TYPE_LINEAR:
-      return RefPtr<LinearGradient>(new LinearGradient(pattern, false /* does not have reference */));
+      return make_refptr_for_instance<LinearGradient>(new LinearGradient(pattern, false /* does not have reference */));
       break;
     case CAIRO_PATTERN_TYPE_RADIAL:
-      return RefPtr<RadialGradient>(new RadialGradient(pattern, false /* does not have reference */));
+      return make_refptr_for_instance<RadialGradient>(new RadialGradient(pattern, false /* does not have reference */));
       break;
     default:
-      return RefPtr<Pattern>(new Pattern(pattern, false /* does not have reference */));
+      return make_refptr_for_instance<Pattern>(new Pattern(pattern, false /* does not have reference */));
   }
 }
 
@@ -704,22 +632,7 @@ RefPtr<const Pattern> Context::get_source() const
 {
   auto pattern = cairo_get_source(const_cast<cobject*>(cobj()));
   check_object_status_and_throw_exception(*this);
-  return RefPtr<const Pattern>::cast_const (get_pattern_wrapper (pattern));
-}
-
-RefPtr<SurfacePattern> Context::get_source_for_surface()
-{
-  auto pattern = cairo_get_source(cobj());
-  check_object_status_and_throw_exception(*this);
-  auto pattern_type = cairo_pattern_get_type(pattern);
-  if (pattern_type != CAIRO_PATTERN_TYPE_SURFACE)
-    return {};
-  return RefPtr<SurfacePattern>(new SurfacePattern(pattern, false /* does not have reference */));
-}
-
-RefPtr<const SurfacePattern> Context::get_source_for_surface() const
-{
-  return const_cast<Context*>(this)->get_source_for_surface();
+  return get_pattern_wrapper(pattern);
 }
 
 double Context::get_tolerance() const
@@ -747,7 +660,7 @@ void Context::get_current_point(double& x, double& y) const
   check_object_status_and_throw_exception(*this);
 }
 
-FillRule Context::get_fill_rule() const
+Context::FillRule Context::get_fill_rule() const
 {
   const auto result = static_cast<FillRule>(cairo_get_fill_rule(const_cast<cobject*>(cobj())));
   check_object_status_and_throw_exception(*this);
@@ -761,14 +674,14 @@ double Context::get_line_width() const
   return result;
 }
 
-LineCap Context::get_line_cap() const
+Context::LineCap Context::get_line_cap() const
 {
   const auto result = static_cast<LineCap>(cairo_get_line_cap(const_cast<cobject*>(cobj())));
   check_object_status_and_throw_exception(*this);
   return result;
 }
 
-LineJoin Context::get_line_join() const
+Context::LineJoin Context::get_line_join() const
 {
   const auto result = static_cast<LineJoin>(cairo_get_line_join(const_cast<cobject*>(cobj())));
   check_object_status_and_throw_exception(*this);
@@ -802,12 +715,6 @@ void Context::get_matrix(Matrix& matrix)
   check_object_status_and_throw_exception(*this);
 }
 
-void Context::get_matrix(cairo_matrix_t& matrix)
-{
-  cairo_get_matrix(cobj(), &matrix);
-  check_object_status_and_throw_exception(*this);
-}
-
 Matrix Context::get_matrix() const
 {
   Cairo::Matrix m;
@@ -823,16 +730,16 @@ RefPtr<Surface> get_surface_wrapper (cairo_surface_t* surface)
   switch (surface_type)
   {
     case CAIRO_SURFACE_TYPE_IMAGE:
-      return RefPtr<ImageSurface>(new ImageSurface(surface, false /* does not have reference */));
+      return make_refptr_for_instance<ImageSurface>(new ImageSurface(surface, false /* does not have reference */));
       break;
 #if CAIRO_HAS_PDF_SURFACE
     case CAIRO_SURFACE_TYPE_PDF:
-      return RefPtr<PdfSurface>(new PdfSurface(surface, false /* does not have reference */));
+      return make_refptr_for_instance<PdfSurface>(new PdfSurface(surface, false /* does not have reference */));
       break;
 #endif
 #if CAIRO_HAS_PS_SURFACE
     case CAIRO_SURFACE_TYPE_PS:
-      return RefPtr<PsSurface>(new PsSurface(surface, false /* does not have reference */));
+      return make_refptr_for_instance<PsSurface>(new PsSurface(surface, false /* does not have reference */));
       break;
 #endif
 #if CAIRO_HAS_XLIB_SURFACE
@@ -842,7 +749,7 @@ RefPtr<Surface> get_surface_wrapper (cairo_surface_t* surface)
 #endif
 #if CAIRO_HAS_GLITZ_SURFACE
     case CAIRO_SURFACE_TYPE_GLITZ:
-      return RefPtr<GlitzSurface>(new GlitzSurface(surface, false /* does not have reference */));
+      return make_refptr_for_instance<GlitzSurface>(new GlitzSurface(surface, false /* does not have reference */));
       break;
 #endif
 #if CAIRO_HAS_QUARTZ_SURFACE
@@ -852,7 +759,7 @@ RefPtr<Surface> get_surface_wrapper (cairo_surface_t* surface)
 #endif
 #if CAIRO_HAS_SCRIPT_SURFACE
     case CAIRO_SURFACE_TYPE_SCRIPT:
-      return RefPtr<ScriptSurface>(new ScriptSurface(surface, false));
+      return make_refptr_for_instance<ScriptSurface>(new ScriptSurface(surface, false));
       break;
 #endif
 #if CAIRO_HAS_WIN32_SURFACE
@@ -862,7 +769,7 @@ RefPtr<Surface> get_surface_wrapper (cairo_surface_t* surface)
 #endif
 #if CAIRO_HAS_SVG_SURFACE
     case CAIRO_SURFACE_TYPE_SVG:
-      return RefPtr<SvgSurface>(new SvgSurface(surface, false /* does not have reference */));
+      return make_refptr_for_instance<SvgSurface>(new SvgSurface(surface, false /* does not have reference */));
       break;
 #endif
     // the following surfaces are not directly supported in cairomm yet
@@ -871,7 +778,7 @@ RefPtr<Surface> get_surface_wrapper (cairo_surface_t* surface)
     case CAIRO_SURFACE_TYPE_BEOS:
     case CAIRO_SURFACE_TYPE_XCB:
     default:
-      return RefPtr<Surface>(new Surface(surface, false /* does not have reference */));
+      return make_refptr_for_instance<Surface>(new Surface(surface, false /* does not have reference */));
   }
 }
 
@@ -886,7 +793,7 @@ RefPtr<const Surface> Context::get_target() const
 {
   auto surface = cairo_get_target(const_cast<cobject*>(cobj()));
   check_object_status_and_throw_exception(*this);
-  return RefPtr<const Surface>::cast_const (get_surface_wrapper (surface));
+  return get_surface_wrapper(surface);
 }
 
 Path* Context::copy_path() const
