@@ -1,6 +1,9 @@
 /* M_PI is defined in math.h in the case of Microsoft Visual C++, Solaris,
  * et. al.
  */
+#if defined(_MSC_VER)
+#define _USE_MATH_DEFINES
+#endif 
 
 #include <string>
 #include <iostream>
@@ -13,7 +16,7 @@
 int main()
 {
     auto surface =
-        Cairo::ImageSurface::create(Cairo::FORMAT_ARGB32, 600, 400);
+        Cairo::ImageSurface::create(Cairo::Surface::Format::ARGB32, 600, 400);
 
     auto cr = Cairo::Context::create(surface);
 
